package org.seamcat.epp;

import java.util.ArrayList;
import java.util.List;

import org.seamcat.model.Scenario;
import org.seamcat.model.eventprocessing.EventProcessingPlugin;
import org.seamcat.model.functions.Point2D;
import org.seamcat.model.generic.GenericSystem;
import org.seamcat.model.plugin.BooleanParameter;
import org.seamcat.model.plugin.DoubleParameter;
import org.seamcat.model.plugin.Parameter;
import org.seamcat.model.plugin.ParameterDefinition;
import org.seamcat.model.simulation.result.EventResult;
import org.seamcat.model.types.result.DoubleResultType;
import org.seamcat.model.types.result.IntegerResultType;
import org.seamcat.model.types.result.ResultTypes;
import org.seamcat.model.types.result.ScatterDiagramResultType;
import org.seamcat.model.types.result.SingleValueTypes;
import org.seamcat.model.types.result.StringResultType;
import org.seamcat.model.types.result.VectorResultType;

public class TestFrameEPP implements EventProcessingPlugin {

	private int eventCount;
	private List<Double> dRSS;
	private boolean isPanelClosed;
	private boolean resultsPassed;
	private boolean exportSelection1;
	private boolean exportSelection2;
	private boolean exportSelection3;
	private List<Double> iRSSunw;
	private double tCI;
	private double sensitivity;
	private boolean closeExternalFrame;

	@Override
	public List<ParameterDefinition> getParameterDefinitions() {
		List<ParameterDefinition> parameters = new ArrayList<ParameterDefinition>();
		/* parameters to be added here */
		parameters.add(new BooleanParameter("close external frame automatically").defaultValue(true));

		return parameters;
	}

	@Override
	public String getDescription() {
		return "Test frame passing results to the EPP";
	}

	@Override
	public ResultTypes evaluate(Scenario scenario, Iterable<EventResult> results, List<Parameter> params) {
		ResultTypes types = new ResultTypes();
		List<SingleValueTypes<?>> singleTypes = types.getSingleValueTypes();
		List<VectorResultType> vectors = types.getVectorResultTypes();
		List<ScatterDiagramResultType> scatterDiagramResultTypes = types.getScatterDiagramResultTypes();

		dRSS = new ArrayList<Double>();
		iRSSunw = new ArrayList<Double>();
		tCI = ((GenericSystem) scenario.getVictimSystem()).getReceiver().getProtectionRatio();
		sensitivity = ((GenericSystem) scenario.getVictimSystem()).getReceiver().getSensitivity();

		initPlugin(scenario, params); // as an option

		eventCount = 0;
		for (EventResult res : results) {
			eventCount++;
			dRSS.add(res.getDRSSResult());
			iRSSunw.add(res.getIRSSUnwantedSummation());
		}

		if (eventCount == scenario.numberOfEvents()) { /* waits for completion of the simulation */

			vectors.add(new VectorResultType("dRSS", "dBm", dRSS));

			TestFrame frame = new TestFrame();
			frame.start(); /* starts the external frame as a new thread */

			proceedExternalPart(scenario, results, vectors, params, singleTypes, scatterDiagramResultTypes);

		}

		return types;
	}

	/**
	 * passes the collected data to the external part of the EPP and waits for
	 * reported results or the closure of the external frame (without results)
	 * 
	 * @param scenario
	 * @param results
	 * @param vectors
	 * @param params
	 * @param scatterDiagramResultTypes
	 * @param singleType
	 */
	private void proceedExternalPart(Scenario scenario, Iterable<EventResult> results, List<VectorResultType> vectors, List<Parameter> params,
			List<SingleValueTypes<?>> singleType, List<ScatterDiagramResultType> scatterDiagramResultTypes) {
		/* all methods called from here are synchronised */
		TestFrameCalc calculator = new TestFrameCalc();
		calculator.setVectors(dRSS, iRSSunw, tCI, sensitivity);
		calculator.initializeResultPanel();
		boolean readyForExport = false;
		do { /* loop waiting for either closing the external frame (without results) or reporting results */
			isPanelClosed = calculator.getIsPanelClosed(); /* set when closing the external frame also without results */
			if (isPanelClosed)
				readyForExport = true;
			resultsPassed = calculator.getResultsAvailable(); /* activated when a result is available */
			if (resultsPassed) { /* checks the available result vectors */
				String msg = "This message and the below results (except dRSS) are generated outside the main part of the EPP.\n\n";
				singleType.add(new StringResultType("message: ", null, msg));
				singleType.add(new StringResultType(" ", null, "This row could be understood as second line of the above message."));
				singleType.add(new StringResultType("Settings ", null, calculator.getSettings()));
				singleType.add(new IntegerResultType("angle as new integer result type", "degree", 345));
				if (calculator.getResultVector1().size() == scenario.numberOfEvents()) {
					vectors.add(new VectorResultType("dRSS 1", "dBm", calculator.getResultVector1()));
				}
				if (calculator.getResultVector2().size() == scenario.numberOfEvents()) {
					vectors.add(new VectorResultType("dRSS 2", "dBm", calculator.getResultVector2()));
				}
				if (calculator.getResultVector3().size() == scenario.numberOfEvents()) {
					vectors.add(new VectorResultType("dRSS 3", "dBm", calculator.getResultVector3()));
				}
				if (calculator.getDistributionCIvalues().size() == scenario.numberOfEvents()) {
					vectors.add(new VectorResultType("C/I - unwanted", "dB", calculator.getDistributionCIvalues()));
				}
				double exceedingLimit = calculator.getExceedingLimit();
				if (exceedingLimit >= 0) {
					singleType.add(new DoubleResultType("exceeding limit of C/I - unwanted", "%", exceedingLimit));
				}
				if (exceedingLimit >= 0) {
					singleType.add(new DoubleResultType("to show a greater number of single result types", "%", exceedingLimit));
				}
				List<Point2D> dCI = calculator.getDistributionCI();
				if (dCI.size() == scenario.numberOfEvents()) {
					ScatterDiagramResultType rFigure = new ScatterDiagramResultType("CDF of C/I - unwanted ", "C/I [dB]", "*100 %");
					for (int k = 0; k < dCI.size(); k++) {
						rFigure.getScatterPoints().add(dCI.get(k));
					}
					scatterDiagramResultTypes.add(rFigure);
				}

				readyForExport = true;

			}
		} while (readyForExport == false);

		if (resultsPassed && closeExternalFrame) {
			calculator.closeExternalFrame();
		}
	}

	/**
	 * personally preferred to configure the plugin
	 * 
	 * @param scenario
	 * @param params
	 */
	private void initPlugin(Scenario scenario, List<Parameter> params) {
		closeExternalFrame = params.get(0).getValue();
	}

	@Override
	public String toString() {
		return "PassResults"; // e.g. TemplateEPP
	}
}
